//  Pusheo
//
//  Created by Inscripts on 19/03/15.
//  Copyright (c) 2015 inscripts. All rights reserved.

#import <Foundation/Foundation.h>
#import <Security/SecCertificate.h>

typedef NS_ENUM(NSInteger, PSReadyState) {
    PS_CONNECTING   = 0,
    PS_OPEN         = 1,
    PS_CLOSING      = 2,
    PS_CLOSED       = 3,
};

typedef enum PSStatusCode : NSInteger {
    PSStatusCodeNormal = 1000,
    PSStatusCodeGoingAway = 1001,
    PSStatusCodeProtocolError = 1002,
    PSStatusCodeUnhandledType = 1003,
    // 1004 reserved.
    PSStatusNoStatusReceived = 1005,
    // 1004-1006 reserved.
    PSStatusCodeInvalidUTF8 = 1007,
    PSStatusCodePolicyViolated = 1008,
    PSStatusCodeMessageTooBig = 1009,
} PSStatusCode;

@class PusheoSocket;

extern NSString *const PusheoSocketErrorDomain;
extern NSString *const PSHTTPResponseErrorKey;

#pragma mark - PusheoSocketDelegate

@protocol PusheoSocketDelegate;

#pragma mark - PusheoSocket

@interface PusheoSocket : NSObject <NSStreamDelegate>

@property (nonatomic, weak) id <PusheoSocketDelegate> delegate;

@property (nonatomic, readonly) PSReadyState readyState;
@property (nonatomic, readonly, retain) NSURL *url;

// This returns the negotiated protocol.
// It will be nil until after the handshake completes.
@property (nonatomic, readonly, copy) NSString *protocol;

// Protocols should be an array of strings that turn into Sec-WebSocket-Protocol.
- (id)initWithURLRequest:(NSURLRequest *)request protocols:(NSArray *)protocols;
- (id)initWithURLRequest:(NSURLRequest *)request;

// Some helper constructors.
- (id)initWithURL:(NSURL *)url protocols:(NSArray *)protocols;
- (id)initWithURL:(NSURL *)url;

// Delegate queue will be dispatch_main_queue by default.
// You cannot set both OperationQueue and dispatch_queue.
- (void)setDelegateOperationQueue:(NSOperationQueue*) queue;
- (void)setDelegateDispatchQueue:(dispatch_queue_t) queue;

// By default, it will schedule itself on +[NSRunLoop SR_networkRunLoop] using defaultModes.
- (void)scheduleInRunLoop:(NSRunLoop *)aRunLoop forMode:(NSString *)mode;
- (void)unscheduleFromRunLoop:(NSRunLoop *)aRunLoop forMode:(NSString *)mode;

// PusheoSockets are intended for one-time-use only.  Open should be called once and only once.
- (void)open;

- (void)close;
- (void)closeWithCode:(NSInteger)code reason:(NSString *)reason;

// Send a UTF8 String or Data.
- (void)send:(id)data;

// Send Data (can be nil) in a ping message.
- (void)sendPing:(NSData *)data;

@end

#pragma mark - PusheoSocketDelegate

@protocol PusheoSocketDelegate <NSObject>

// message will either be an NSString if the server is using text
// or NSData if the server is using binary.
- (void)webSocket:(PusheoSocket *)webSocket didReceiveMessage:(id)message;

@optional

- (void)webSocketDidOpen:(PusheoSocket *)webSocket;
- (void)webSocket:(PusheoSocket *)webSocket didFailWithError:(NSError *)error;
- (void)webSocket:(PusheoSocket *)webSocket didCloseWithCode:(NSInteger)code reason:(NSString *)reason wasClean:(BOOL)wasClean;
- (void)webSocket:(PusheoSocket *)webSocket didReceivePong:(NSData *)pongPayload;

@end

#pragma mark - NSURLRequest (CertificateAdditions)

@interface NSURLRequest (CertificateAdditions)

@property (nonatomic, retain, readonly) NSArray *SR_SSLPinnedCertificates;

@end

#pragma mark - NSMutableURLRequest (CertificateAdditions)

@interface NSMutableURLRequest (CertificateAdditions)

@property (nonatomic, retain) NSArray *SR_SSLPinnedCertificates;

@end

#pragma mark - NSRunLoop (PusheoSocket)

@interface NSRunLoop (PusheoSocket)

+ (NSRunLoop *)PS_networkRunLoop;

@end
